# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['apodo']

package_data = \
{'': ['*']}

install_requires = \
['cython>=0.29.12,<0.30.0']

setup_kwargs = {
    'name': 'apodo',
    'version': '0.1.0.dev0',
    'description': 'A remarkably fast asynchronous web framework for Python.',
    'long_description': '# Apodo\nThe async Python web framework that acts fast and lands on its feet.\n\nApodo is an asynchronous Python web framework that derives from the Vibora framework, implementing Cython and other performance-enhancing technologies to significantly increase processing speed and efficiency.\n\n## Framework Goals\n- Go quick\n- Run clean\n- Stay simple\n\n## Project Goals\n- Work together\n- Generate and support cool ideas\n- Learn and grow as a team\n\n"Your team\'s strength is not a function of the talent of individual members. It\'s a function of their collaboration, tenacity, and mutual respect."\n\n## Development Plan\nApodo, while derived from the Vibora framework, will be built with a new design philosophy in mind. We will reference the old source of Vibora as we build, but only for some lower-level concepts.\n\nA major pitfall of Vibora was its lack of strict system design guidelines, as well of its lack of documentation. Solid design is to be held in high regard along with functional speed, and design should not and will not be compromised for functional speed. Building a spaghetti dinner that runs very quickly is fine, but building a well-designed, fast, and scalable web framework is what we\'re more interested in.\n\nThe plan for Apodo is to start with base functionality, and iteratively add features, with Vibora as a functional reference.\n\n## Development Pipeline\n\nAs one of our goals is to work together, the development pipeline is structured knowing that all may not be comfortable/confident enough with Cython and lower-level concepts to be able to contribute to the core system.\n\n### Vibora Analysis\nThis project is for those mentioned above. As the core team works, it is necessary that we are able to reference Vibora. Vibora is woefully lacking in its inline and online documentation, and a way for newcomers to contribute is by digging through the source in the `legacy` package, figuring out what it does, and writing in docstrings/refactoring illegible code. This process will not only support and speed up the development of Apodo, but also equip newcomers with tons of knowledge about how this type of web framework operates under the hood.\n\nTo contribute in this way, all you have to do is fork with the branch schema of `v-rewrite-<module or package name>` and submit a PR once you\'re done adding docstrings and refactoring illegible code. Your PR must pass one review before it is merged to `dev`.\n\n### Core Development\nThis project is for those experienced in Python, with knowledge of how it works as a language, as well as C concepts like Cython and other C-optimised practices and libraries, as well as sufficient understanding of web technologies and practices.\n\nIn order to contribute this way, you must be in the Apodo Slack group and approved as a contributor on the core repository.\n\nContribution to the core system will be heavily discussion-based until we release a first iteration, after which the iterative feature adds will begin. Until then, contact Elliott about joining the core team, and we\'ll go from there.\n\n## Development Guidelines\n\nBe aware that we utilize git hooks to automate and standardize some of the development process. We will conform to our written configurations of the `black` formatter, and of the `flake8` linter.\n\n### Comments/docstrings\n\nAs it stands, commenting and style consistency is woefully lacking in the Vibora library, and those are two things being addressed in this port before we begin work on features/fixes. We will use `black` for formatting, and comment styling will be as follows.\n\nFor module docstrings, comments should look like this:\n\n    """\n    apodo.utils.module (modular path)\n    ~~~~~~~~~~~~~~~~~\n\n    This module implements the `Class` class, and other stuff. (module description)\n    """\n\nFor class docstrings, comments should look like this:\n\n    """ Implements the `Class` class. (short description)\n\n    This class does things that it does. We\'ve written it to do\n    actions and carry out tasks. (long description)\n\n    :param `*args`: These are some arguments.\n    :param `**kwargs`: These are some keyword arguments. (parameters)\n    """\n\nFor method docstrings, comments should look like this:\n\n    """ Does an action. (short description)\n\n    This method does things and stuff. Note that it does things\n    in a certain way as of version 0.1.0. (long description)\n\n    :param `thing`: A thing (`str`) with which to do stuff.\n    :param `stuff`: (optional) Some stuff (`dict`) with which to do things. (parameters w/ type intentions)\n\n    :return `product`: A `Product` object. (return w/ type intention)\n    """\n\nOther one-line commenting should be kept to a mininum but used effectively and concisely when necessary.\n\n#### Soft Rules:\n- Put backticks (``) around object and variable names.\n- Wrap comments to 88 characters.\n- Capitalize the first letters of parameter and return descriptions.\n\n### Typing\n\nWe will *not* use type hints. While helpful to an extent, type hints in Python dirty code significantly, and typing can be notated in docstrings. So, the parameter and return lines in all method docstrings MUST denote the intended type of the given variables.\n\n## Let\'s build together.\nThe Vibora framework, while intelligently conceptualized and designed, lacked the community and structure that an open-source project needs to thrive. While a lot of brilliant engineers have created great open-source software mostly on their own, it\'s more meaningful and enriching for all involved when it becomes a community effort.\n\nA major goal of this project is to cultivate an efficient and involved development/contribution pipeline, rather than an isolated and centralized dependent workflow for few.\n\nThis project needs developers! Join us on the [slack channel](https://join.slack.com/t/apodoproject/shared_invite/enQtNzA1NjcwMDU4MDA2LWIyZWFmNDY2YzEyM2RmYWQ2OWM3MzQyN2QwYzllYzg3OGRhMzJkOWIwMjA2OTEyOGVkYTliZTA4OWQwMDI1Y2U)\n',
    'author': 'Elliott Maguire',
    'author_email': 'me@elliott-m.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
